import { Injectable } from '@angular/core';

import { Actions, ofType, createEffect } from '@ngrx/effects';

import { Store } from '@ngrx/store';

import { NGXLogger } from 'ngx-logger';
import { catchError, exhaustMap, map } from 'rxjs/operators';

import { of } from 'rxjs';

import {
  userPasswordSet,
  userPasswordSetSuccess,
  userPasswordSetFailure
} from './actions';
import {
  ServiceProtocolService,
  UserPasswordSetResponse
} from '@ucap-webmessenger/protocol-service';

@Injectable()
export class Effects {
  userPasswordSet$ = createEffect(() =>
    this.actions$.pipe(
      ofType(userPasswordSet),
      map(action => action.req),
      exhaustMap(req =>
        this.serviceProtocolService.userPasswordSet(req).pipe(
          map((res: UserPasswordSetResponse) => {
            return userPasswordSetSuccess({
              res
            });
          }),
          catchError(error => of(userPasswordSetFailure({ error })))
        )
      )
    )
  );

  constructor(
    private actions$: Actions,
    private serviceProtocolService: ServiceProtocolService,
    private store: Store<any>,
    private logger: NGXLogger
  ) {}
}
