import {
  Component,
  OnInit,
  Input,
  Output,
  EventEmitter,
  ViewChild,
  ElementRef,
  ChangeDetectorRef
} from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Company } from '@ucap-webmessenger/api-external';
import { LocalStorageService } from '@ucap-webmessenger/web-storage';
import { LoginInfo, KEY_LOGIN_INFO } from '@app/types';

@Component({
  selector: 'ucap-account-change-password',
  templateUrl: './change-password.component.html',
  styleUrls: ['./change-password.component.scss']
})
export class ChangePasswordComponent implements OnInit {
  @Input()
  changePasswordBtnEnable: boolean;

  @Output()
  changePassword = new EventEmitter<{
    currentLoginPw: string;
    newLoginPw: string;
    newConfirmLoginPw: string;
    notValid: () => void;
  }>();

  @ViewChild('currentLoginPw', { static: true })
  currentLoginPwElementRef: ElementRef;
  @ViewChild('newLoginPw', { static: true })
  newLoginPwElementRef: ElementRef;

  changePasswordForm: FormGroup;

  constructor(
    private formBuilder: FormBuilder,
    private changeDetectorRef: ChangeDetectorRef,
    private localStorageService: LocalStorageService
  ) {}

  ngOnInit() {
    const loginInfo: LoginInfo = this.localStorageService.get<LoginInfo>(
      KEY_LOGIN_INFO
    );
    this.changePasswordForm = this.formBuilder.group({
      currentLoginPw: ['', Validators.required],
      newLoginPw: ['', Validators.required],
      newConfirmLoginPw: ['', Validators.required]
    });
  }

  onClickChangePassword() {
    this.changePassword.emit({
      currentLoginPw: this.changePasswordForm.get('currentLoginPw').value,
      newLoginPw: this.changePasswordForm.get('newLoginPw').value,
      newConfirmLoginPw: this.changePasswordForm.get('newConfirmLoginPw').value,
      notValid: () => {
        this.currentLoginPwElementRef.nativeElement.focus();
      }
    });
  }
}
